from marshmallow import Schema, fields, post_load

from workspaces.workflow.enum import WorkflowEventType
from workspaces.workflow.schema import WorkflowEvent


class WorkflowEventSchema(Schema):
    """
    Default schema for serializing WorkflowEvents.
    """

    workflow_request_id = fields.Integer()
    job_name = fields.String()
    condor_job_id = fields.String()
    event_type = fields.Method("get_type", deserialize="load_type")
    timestamp = fields.String()
    log = fields.String()
    retval = fields.Integer(allow_none=True)

    # Enums apparently are not a first-class field type in Marshmallow
    def get_type(self, obj: WorkflowEventType) -> str:
        return obj.type.name.lower()

    def load_type(self, name: str) -> WorkflowEventType:
        return next(et for et in WorkflowEventType if et.name.lower() == name)

    @post_load
    def make_event(self, data, **kwargs):
        return WorkflowEvent(**data)


workflow_event = WorkflowEventSchema()
